window.Gleam.initSearch([{"doc":"gleam_package_interface","title":"gleam_package_interface","content":"# gleam_package_interface\n\n[![Package Version](https://img.shields.io/hexpm/v/gleam_package_interface)](https://hex.pm/packages/gleam_package_interface)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/gleam_package_interface/)\n![Supported targets](https://img.shields.io/badge/supports-all_targets-ffaff3)\n\n## Installation\n\nAdd `gleam_package_interface` to your Gleam project:\n\n```sh\ngleam add gleam_package_interface\n```\n\n## What&#39;s a package interface?\n\nWhenever you build your project&#39;s documentation with `gleam docs build`, the\nGleam compiler will also produce a handy json file\n`./build/dev/docs/&lt;your package&gt;/package-interface.json`\ncontaining data about your package: that&#39;s the package interface.\n\nIt has all public information your package exposes to the outside world: type\ndefinitions, type aliases, public functions and constants — each annotated with\nits own type and with its documentation.\n\n&gt; You can also have the compiler build the package interface using the\n&gt; `gleam export package-interface` command.\n\nLet&#39;s have a look at a small example. Imagine you have a module called `wibble`\nwith the following definition:\n\n```gleam\n/// Documentation!\npub fn wibbler(label n: Int) -&gt; Int {\n  todo\n}\n```\n\nThe resulting package interface will look something like this (some keys where\nomitted to keep the example short):\n\n```json\n{\n  &quot;package&quot;: &quot;your package&quot;,\n  &quot;modules&quot;: {\n    &quot;wibble&quot;: {\n      &quot;functions&quot;: {\n        &quot;wibbler&quot;: {\n          &quot;documentation&quot;: &quot;Documentation!\\n&quot;,\n          &quot;parameters&quot;: [\n            {\n              &quot;label&quot;: &quot;label&quot;,\n              &quot;type&quot;: {\n                &quot;kind&quot;: &quot;named&quot;,\n                &quot;package&quot;: &quot;&quot;,\n                &quot;module&quot;: &quot;gleam&quot;,\n                &quot;name&quot;: &quot;Int&quot;\n              }\n            }\n          ],\n          &quot;return&quot;: {\n            &quot;kind&quot;: &quot;named&quot;,\n            &quot;package&quot;: &quot;&quot;,\n            &quot;module&quot;: &quot;gleam&quot;,\n            &quot;name&quot;: &quot;Int&quot;\n          }\n        }\n      }\n    }\n  }\n}\n```\n\nTo get a proper feel of the structure of the generated package interface you can\nhave a look at this\n[package&#39;s types](https://hexdocs.pm/gleam_package_interface/).\n\n## Usage\n\nThis package provides Gleam types to describe a package interface and a\n`decoder` to decode the json package interface into a Gleam value.\n\n```gleam\n// gleam add gleam_json\n// gleam add simplifile\nimport gleam/json\nimport gleam/package_interface\nimport simplifile\n\npub fn main() {\n  let assert Ok(json) = simplifile.read(&quot;path to the package interface&quot;)\n  let assert Ok(interface) = json.decode(json, using: package_interface.decoder)\n  todo as &quot;now you can use the package interface however you want!&quot;\n}\n```\n","url":"index.html"},{"doc":"gleam/package_interface","title":"Constant","content":"pub type Constant {\n  Constant(\n    documentation: Option(String),\n    deprecation: Option(Deprecation),\n    implementations: Implementations,\n    type_: Type,\n  )\n}\n A Gleam constant.\n\n \n pub const my_favourite_number = 11\n \n\nConstant(\n  documentation: Option(String),\n  deprecation: Option(Deprecation),\n  implementations: Implementations,\n  type_: Type,\n)\n\ndocumentation\n&lt;p&gt;The constant’s documentation comment (that is every line preceded by\n&lt;code&gt;///&lt;/code&gt;).&lt;/p&gt;\n\ndeprecation\n&lt;p&gt;If the constant is deprecated this will hold the reason of the\ndeprecation.&lt;/p&gt;\n\nSynonyms:\ngleam/package_interface.Constant\ngleam/package_interface Constant","url":"gleam/package_interface.html#Constant"},{"doc":"gleam/package_interface","title":"Deprecation","content":"pub type Deprecation {\n  Deprecation(message: String)\n}\n A deprecation notice that can be added to definition using the\n `@deprecated` annotation.\n\nDeprecation(message: String)\n\n\nSynonyms:\ngleam/package_interface.Deprecation\ngleam/package_interface Deprecation","url":"gleam/package_interface.html#Deprecation"},{"doc":"gleam/package_interface","title":"Function","content":"pub type Function {\n  Function(\n    documentation: Option(String),\n    deprecation: Option(Deprecation),\n    implementations: Implementations,\n    parameters: List(Parameter),\n    return: Type,\n  )\n}\n A Gleam function definition.\n\n \n pub fn reverse(list: List(a)) -&gt; List(a) { todo }\n \nFunction(\n  documentation: Option(String),\n  deprecation: Option(Deprecation),\n  implementations: Implementations,\n  parameters: List(Parameter),\n  return: Type,\n)\n\ndocumentation\n&lt;p&gt;The function’s documentation comment (that is every line preceded by\n&lt;code&gt;///&lt;/code&gt;).&lt;/p&gt;\n\ndeprecation\n&lt;p&gt;If the function is deprecated this will hold the reason of the\ndeprecation.&lt;/p&gt;\n\nSynonyms:\ngleam/package_interface.Function\ngleam/package_interface Function","url":"gleam/package_interface.html#Function"},{"doc":"gleam/package_interface","title":"Implementations","content":"pub type Implementations {\n  Implementations(\n    gleam: Bool,\n    uses_erlang_externals: Bool,\n    uses_javascript_externals: Bool,\n  )\n}\n Metadata about how a value is implemented and the targets it supports.\n\nImplementations(\n  gleam: Bool,\n  uses_erlang_externals: Bool,\n  uses_javascript_externals: Bool,\n)\n\ngleam\n&lt;p&gt;Set to &lt;code&gt;True&lt;/code&gt; if the const/function has a pure Gleam implementation\n(that is, it never uses external code).\nBeing pure Gleam means that the function will support all Gleam\ntargets, even future ones that are not present to this day.&lt;/p&gt;\n&lt;p&gt;Consider the following function:&lt;/p&gt;\n&lt;pre&gt;&lt;code class=&quot;language-gleam&quot;&gt;@external(erlang, &quot;foo&quot;, &quot;bar&quot;)\npub fn a_random_number() -&amp;gt; Int {\n  4\n  // This is a default implementation.\n}\n&lt;/code&gt;&lt;/pre&gt;\n&lt;p&gt;The implementations for this function will look like this:&lt;/p&gt;\n&lt;pre&gt;&lt;code class=&quot;language-gleam&quot;&gt;Implementations(\n  gleam: True,\n  uses_erlang_externals: True,\n  uses_javascript_externals: False,\n)\n&lt;/code&gt;&lt;/pre&gt;\n&lt;ul&gt;\n&lt;li&gt;&lt;code&gt;gleam: True&lt;/code&gt; means that the function has a pure Gleam implementation\nand thus it can be used on all Gleam targets with no problems.&lt;/li&gt;\n&lt;li&gt;&lt;code&gt;uses_erlang_externals: True&lt;/code&gt; means that the function will use Erlang\nexternal code when compiled to the Erlang target.&lt;/li&gt;\n&lt;li&gt;&lt;code&gt;uses_javascript_externals: False&lt;/code&gt; means that the function won’t use\nJavaScript external code when compiled to JavaScript. The function can\nstill be used on the JavaScript target since it has a pure Gleam\nimplementation.&lt;/li&gt;\n&lt;/ul&gt;\n\nuses_erlang_externals\n&lt;p&gt;Set to &lt;code&gt;True&lt;/code&gt; if the const/function is defined using Erlang external\ncode. That means that the function will use Erlang code through FFI when\ncompiled for the Erlang target.&lt;/p&gt;\n\nuses_javascript_externals\n&lt;p&gt;Set to &lt;code&gt;True&lt;/code&gt; if the const/function is defined using JavaScript external\ncode. That means that the function will use JavaScript code through FFI\nwhen compiled for the JavaScript target.&lt;/p&gt;\n&lt;p&gt;Let’s have a look at an example:&lt;/p&gt;\n&lt;pre&gt;&lt;code class=&quot;language-gleam&quot;&gt;@external(javascript, &quot;foo&quot;, &quot;bar&quot;)\npub fn javascript_only() -&amp;gt; Int\n&lt;/code&gt;&lt;/pre&gt;\n&lt;p&gt;It’s implementations field will look like this:&lt;/p&gt;\n&lt;pre&gt;&lt;code class=&quot;language-gleam&quot;&gt;Implementations(\n  gleam: False,\n  uses_erlang_externals: False,\n  uses_javascript_externals: True,\n)\n&lt;/code&gt;&lt;/pre&gt;\n&lt;ul&gt;\n&lt;li&gt;&lt;code&gt;gleam: False&lt;/code&gt; means that the function doesn’t have a pure Gleam\nimplementations. This means that the function is only defined using\nexternals and can only be used on some targets.&lt;/li&gt;\n&lt;li&gt;&lt;code&gt;uses_erlang_externals: False&lt;/code&gt; the function is not using external\nErlang code. So, since the function doesn’t have a fallback pure Gleam\nimplementation, you won’t be able to compile it on this target.&lt;/li&gt;\n&lt;li&gt;&lt;code&gt;uses_javascript_externals: True&lt;/code&gt; the function is using JavaScript\nexternal code. This means that you will be able to use it on the\nJavaScript target with no problems.&lt;/li&gt;\n&lt;/ul&gt;\n\nSynonyms:\ngleam/package_interface.Implementations\ngleam/package_interface Implementations","url":"gleam/package_interface.html#Implementations"},{"doc":"gleam/package_interface","title":"Module","content":"pub type Module {\n  Module(\n    documentation: List(String),\n    type_aliases: Dict(String, TypeAlias),\n    types: Dict(String, TypeDefinition),\n    constants: Dict(String, Constant),\n    functions: Dict(String, Function),\n  )\n}\n A Gleam module.\n\nModule(\n  documentation: List(String),\n  type_aliases: Dict(String, TypeAlias),\n  types: Dict(String, TypeDefinition),\n  constants: Dict(String, Constant),\n  functions: Dict(String, Function),\n)\n\ndocumentation\n&lt;p&gt;All the lines composing the module’s documentation (that is every line\npreceded by a &lt;code&gt;////&lt;/code&gt;).&lt;/p&gt;\n\ntype_aliases\n&lt;p&gt;The public type aliases defined in the module.&lt;/p&gt;\n\ntypes\n&lt;p&gt;The public custom types defined in the module.&lt;/p&gt;\n\nconstants\n&lt;p&gt;The public constants defined in the module.&lt;/p&gt;\n\nfunctions\n&lt;p&gt;The public functions defined in the module.&lt;/p&gt;\n\nSynonyms:\ngleam/package_interface.Module\ngleam/package_interface Module","url":"gleam/package_interface.html#Module"},{"doc":"gleam/package_interface","title":"Package","content":"pub type Package {\n  Package(\n    name: String,\n    version: String,\n    gleam_version_constraint: Option(String),\n    modules: Dict(String, Module),\n  )\n}\n A Gleam package.\n\nPackage(\n  name: String,\n  version: String,\n  gleam_version_constraint: Option(String),\n  modules: Dict(String, Module),\n)\n\ngleam_version_constraint\n&lt;p&gt;The Gleam version constraint that the package specifies in its\n&lt;code&gt;gleam.toml&lt;/code&gt;.&lt;/p&gt;\n\nSynonyms:\ngleam/package_interface.Package\ngleam/package_interface Package","url":"gleam/package_interface.html#Package"},{"doc":"gleam/package_interface","title":"Parameter","content":"pub type Parameter {\n  Parameter(label: Option(String), type_: Type)\n}\n A parameter (that might be labelled) of a module function or type\n constructor.\n\n \n pub fn map(over list: List(a), with fun: fn(a) -&gt; b) -&gt; b { todo }\n //         ^^^^^^^^^^^^^^^^^^ A labelled parameter.\n \n\nParameter(label: Option(String), type_: Type)\n\n\nSynonyms:\ngleam/package_interface.Parameter\ngleam/package_interface Parameter","url":"gleam/package_interface.html#Parameter"},{"doc":"gleam/package_interface","title":"Type","content":"pub type Type {\n  Tuple(elements: List(Type))\n  Fn(parameters: List(Type), return: Type)\n  Variable(id: Int)\n  Named(\n    name: String,\n    package: String,\n    module: String,\n    parameters: List(Type),\n  )\n}\n A Gleam type.\n\nTuple(elements: List(Type))\n A tuple type like `#(Int, Float)`.\n\n\nFn(parameters: List(Type), return: Type)\n A function type like `fn(Int, a) -&gt; List(a)`.\n\n\nVariable(id: Int)\n A type variable.\n\n \n pub fn foo(value: a) -&gt; a { todo }\n //                ^ This is a type variable.\n \n\n\nNamed(\n  name: String,\n  package: String,\n  module: String,\n  parameters: List(Type),\n)\n A custom named type.\n \n let value: Bool = True\n //         ^^^^ Bool is a named type coming from Gleam&#39;s prelude\n \n\n All prelude types - like Bool, String, etc. - are named types as well.\n In that case, their package is an empty string `&quot;&quot;` and their module\n name is the string `&quot;gleam&quot;`.\n\npackage\n&lt;p&gt;The package the type comes from.&lt;/p&gt;\n\nmodule\n&lt;p&gt;The module the type is defined in.&lt;/p&gt;\n\nparameters\n&lt;p&gt;The concrete type’s type parameters\n.&lt;/p&gt;\n&lt;pre&gt;&lt;code class=&quot;language-gleam&quot;&gt;let result: Result(Int, e) = Ok(1)\n//                ^^^^^^^^ The `Result` named type has 2 parameters.\n//                         In this case it&#39;s the `Int` type and a\n//                         type variable.\n&lt;/code&gt;&lt;/pre&gt;\n\nSynonyms:\ngleam/package_interface.Type\ngleam/package_interface Type","url":"gleam/package_interface.html#Type"},{"doc":"gleam/package_interface","title":"TypeAlias","content":"pub type TypeAlias {\n  TypeAlias(\n    documentation: Option(String),\n    deprecation: Option(Deprecation),\n    parameters: Int,\n    alias: Type,\n  )\n}\n A Gleam type alias.\n\n \n // This is a type alias.\n type Ints = List(Int)\n \n\nTypeAlias(\n  documentation: Option(String),\n  deprecation: Option(Deprecation),\n  parameters: Int,\n  alias: Type,\n)\n\ndocumentation\n&lt;p&gt;The type alias’ documentation comment (that is every line preceded by\n&lt;code&gt;///&lt;/code&gt;).&lt;/p&gt;\n\ndeprecation\n&lt;p&gt;If the type alias is deprecated this will hold the reason of the\ndeprecation.&lt;/p&gt;\n\nparameters\n&lt;p&gt;The number of type variables of the type alias.&lt;/p&gt;\n&lt;pre&gt;&lt;code class=&quot;language-gleam&quot;&gt;type Results(a, b) = List(Result(a, b))\n//   ^^^^^^^^^^^^^ This type alias has 2 type variables.\n\ntype Ints = List(Int)\n//   ^^^^ This type alias has 0 type variables.\n&lt;/code&gt;&lt;/pre&gt;\n\nalias\n&lt;p&gt;The aliased type.&lt;/p&gt;\n&lt;pre&gt;&lt;code class=&quot;language-gleam&quot;&gt;type Ints = List(Int)\n//          ^^^^^^^^^ This is the aliased type.\n&lt;/code&gt;&lt;/pre&gt;\n\nSynonyms:\ngleam/package_interface.TypeAlias\ngleam/package_interface TypeAlias","url":"gleam/package_interface.html#TypeAlias"},{"doc":"gleam/package_interface","title":"TypeConstructor","content":"pub type TypeConstructor {\n  TypeConstructor(\n    documentation: Option(String),\n    name: String,\n    parameters: List(Parameter),\n  )\n}\n A Gleam type constructor.\n\n \n type Result(a, b) {\n   Ok(a)\n   Error(b)\n }\n // `Ok` and `Error` are the type constructors\n // of the `Error` type.\n \n\nTypeConstructor(\n  documentation: Option(String),\n  name: String,\n  parameters: List(Parameter),\n)\n\ndocumentation\n&lt;p&gt;The type constructor’s documentation comment (that is every line\npreceded by &lt;code&gt;///&lt;/code&gt;).&lt;/p&gt;\n\nparameters\n&lt;p&gt;The parameters required by the constructor.&lt;/p&gt;\n&lt;pre&gt;&lt;code class=&quot;language-gleam&quot;&gt;type Box(a) {\n  Box(content: a)\n//    ^^^^^^^^^^ The `Box` constructor has a single\n//               labelled argument.\n}\n&lt;/code&gt;&lt;/pre&gt;\n\nSynonyms:\ngleam/package_interface.TypeConstructor\ngleam/package_interface TypeConstructor","url":"gleam/package_interface.html#TypeConstructor"},{"doc":"gleam/package_interface","title":"TypeDefinition","content":"pub type TypeDefinition {\n  TypeDefinition(\n    documentation: Option(String),\n    deprecation: Option(Deprecation),\n    parameters: Int,\n    constructors: List(TypeConstructor),\n  )\n}\n A Gleam custom type definition.\n\n \n // This is a custom type definition.\n pub type Result(a, b) {\n   Ok(a)\n   Error(b)\n }\n \n\nTypeDefinition(\n  documentation: Option(String),\n  deprecation: Option(Deprecation),\n  parameters: Int,\n  constructors: List(TypeConstructor),\n)\n\ndocumentation\n&lt;p&gt;The type definition’s documentation comment (that is every line preceded\nby &lt;code&gt;///&lt;/code&gt;).&lt;/p&gt;\n\ndeprecation\n&lt;p&gt;If the type definition is deprecated this will hold the reason of the\ndeprecation.&lt;/p&gt;\n\nparameters\n&lt;p&gt;The number of type variables of the type definition.&lt;/p&gt;\n&lt;pre&gt;&lt;code class=&quot;language-gleam&quot;&gt;type Result(a, b) { ... }\n//   ^^^^^^^^^^^^ This type definition has 2 type variables.\n\ntype Person { ... }\n//   ^^^^^^ This type alias has 0 type variables.\n&lt;/code&gt;&lt;/pre&gt;\n\nconstructors\n&lt;p&gt;The type constructors. If the type is opaque this list will be empty as\nthe type doesn’t have any public constructor.&lt;/p&gt;\n&lt;pre&gt;&lt;code class=&quot;language-gleam&quot;&gt;type Result(a, b) {\n  Ok(a)\n  Error(b)\n}\n// `Ok` and `Error` are the type constructors\n// of the `Error` type.\n&lt;/code&gt;&lt;/pre&gt;\n\nSynonyms:\ngleam/package_interface.TypeDefinition\ngleam/package_interface TypeDefinition","url":"gleam/package_interface.html#TypeDefinition"},{"doc":"gleam/package_interface","title":"constant_decoder","content":"pub fn constant_decoder(\n  dynamic: Dynamic,\n) -&gt; Result(Constant, List(DecodeError))\n\nSynonyms:\ngleam/package_interface.constant_decoder\ngleam/package_interface constant_decoder","url":"gleam/package_interface.html#constant_decoder"},{"doc":"gleam/package_interface","title":"constructor_decoder","content":"pub fn constructor_decoder(\n  dynamic: Dynamic,\n) -&gt; Result(TypeConstructor, List(DecodeError))\n\nSynonyms:\ngleam/package_interface.constructor_decoder\ngleam/package_interface constructor_decoder","url":"gleam/package_interface.html#constructor_decoder"},{"doc":"gleam/package_interface","title":"decoder","content":"pub fn decoder(\n  dynamic: Dynamic,\n) -&gt; Result(Package, List(DecodeError))\n\nSynonyms:\ngleam/package_interface.decoder\ngleam/package_interface decoder","url":"gleam/package_interface.html#decoder"},{"doc":"gleam/package_interface","title":"deprecation_decoder","content":"pub fn deprecation_decoder(\n  dynamic: Dynamic,\n) -&gt; Result(Deprecation, List(DecodeError))\n\nSynonyms:\ngleam/package_interface.deprecation_decoder\ngleam/package_interface deprecation_decoder","url":"gleam/package_interface.html#deprecation_decoder"},{"doc":"gleam/package_interface","title":"function_decoder","content":"pub fn function_decoder(\n  dynamic: Dynamic,\n) -&gt; Result(Function, List(DecodeError))\n\nSynonyms:\ngleam/package_interface.function_decoder\ngleam/package_interface function_decoder","url":"gleam/package_interface.html#function_decoder"},{"doc":"gleam/package_interface","title":"implementations_decoder","content":"pub fn implementations_decoder(\n  dynamic: Dynamic,\n) -&gt; Result(Implementations, List(DecodeError))\n\nSynonyms:\ngleam/package_interface.implementations_decoder\ngleam/package_interface implementations_decoder","url":"gleam/package_interface.html#implementations_decoder"},{"doc":"gleam/package_interface","title":"module_decoder","content":"pub fn module_decoder(\n  dynamic: Dynamic,\n) -&gt; Result(Module, List(DecodeError))\n\nSynonyms:\ngleam/package_interface.module_decoder\ngleam/package_interface module_decoder","url":"gleam/package_interface.html#module_decoder"},{"doc":"gleam/package_interface","title":"parameter_decoder","content":"pub fn parameter_decoder(\n  dynamic: Dynamic,\n) -&gt; Result(Parameter, List(DecodeError))\n\nSynonyms:\ngleam/package_interface.parameter_decoder\ngleam/package_interface parameter_decoder","url":"gleam/package_interface.html#parameter_decoder"},{"doc":"gleam/package_interface","title":"type_alias_decoder","content":"pub fn type_alias_decoder(\n  dynamic: Dynamic,\n) -&gt; Result(TypeAlias, List(DecodeError))\n\nSynonyms:\ngleam/package_interface.type_alias_decoder\ngleam/package_interface type_alias_decoder","url":"gleam/package_interface.html#type_alias_decoder"},{"doc":"gleam/package_interface","title":"type_decoder","content":"pub fn type_decoder(\n  dynamic: Dynamic,\n) -&gt; Result(Type, List(DecodeError))\n\nSynonyms:\ngleam/package_interface.type_decoder\ngleam/package_interface type_decoder","url":"gleam/package_interface.html#type_decoder"},{"doc":"gleam/package_interface","title":"type_definition_decoder","content":"pub fn type_definition_decoder(\n  dynamic: Dynamic,\n) -&gt; Result(TypeDefinition, List(DecodeError))\n\nSynonyms:\ngleam/package_interface.type_definition_decoder\ngleam/package_interface type_definition_decoder","url":"gleam/package_interface.html#type_definition_decoder"},{"doc":"gleam/package_interface","title":"gleam/package_interface","content":"","url":"gleam/package_interface.html"}]);